#!/usr/bin/env python

"""Setup script for the CDDB module distribution."""

__revision__ = "$Id: setup.py,v 1.4 2001/03/10 22:34:03 che_fox Exp $"

from distutils.core import setup, Extension

setup (# Distribution meta-data
       name = "CDDB",
       version = "1.3",
       description = "Module for retrieving track information about audio CDs from CDDB",
       author = "Ben Gertzfield",
       author_email = "che@debian.org",
       url = "http://csl.cse.ucsc.edu/~ben/python/",

       # Description of the modules and packages in the distribution
       py_modules = ['CDDB', 'DiscID'],
       ext_modules = [ Extension('cdrom', ['unix/cdrommodule.c']) ]
      )
